/*****************************************************
This program was produced by the
CodeWizardAVR V2.04.4a Advanced

Project :  ( . RTC)      ( +- 59 .)
          + 8         
          +  ( 2-  DS1820 / DS18S20 / DS18B20)
          + LCD 16x2 Winstar 

Date    : 29.06.2010
Author  : Danil Pervakov

Comments:    DS18x20 (   int  float),
          DS1307 ( -  -    
          - ), LCD (        
            )

           : " " -  
                           " "  - 
                           " " -   
                           " " -    
                           "" -
                           "Escape" -  

Chip type           : ATmega8
Program type        : Application
Clock frequency     : 4,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/
/*
Program size: 3983 words (7966 bytes), 97,2% of FLASH
*/

// ----------------------------------------------------------------
//
//  Fuse bit's SKSEL3..0 = 0011, SUT1..0 = 10 (Int RC = 4 MHz) 
//
//   DS18B20 - (PORTB bit 1)
//  
//  Realtime DS1307 - (TWI sda -  27, scl -  28)
//
//   -  (ADC0 -  23) - "L", "R", "U", "D", "Menu", "Esc"
//            (PORTB bit 0) - "" 
//
//  LCD - PORTD (bit 3 - )
//
//  LCD 16x2 (  )
//  WinStar WH1602C-YGH-CTK
//            bit0 - RS  (pin4 LCD)
//            bit1 - R/W (pin5 LCD)
//            bit2 - E   (pin6 LCD)
//            bit3 - 
//            bit4 - DB4 (pin11 LCD)
//            bit5 - DB5 (pin12 LCD)
//            bit6 - DB6 (pin13 LCD)
//            bit7 - DB7 (pin14 LCD)
//

//  -          ,    IBM PC XT   Intel 8088 
#include <mega8.h>

// 1 Wire Bus functions (PORTB bit 1)
#asm
   .equ __w1_port=0x18
   .equ __w1_bit=1
#endasm

// DS18b20 Temperature Sensor functions
#include "ds18x20_v3.h"

// DS1307 Real Time Clock functions TWI (I2C - ) 
#include "ds1307_twi.h"

// Alphanumeric LCD Module functions (PORTD)
#asm
   .equ __lcd_port=0x12
#endasm

// LCD Module functions
#include "lcd_rus.h"

#include <delay.h> 
#include <bcd.h>

// ,   . 
#include "term_clock_mega8_lcd.h"

//  
#include "skin.h"

// -----------------------------------------------------------------------------------------
// Timer 0 overflow interrupt service routine
// 500.000 Hz / 250 = 2.000 Hz
interrupt [TIM0_OVF] void timer0_ovf_isr(void) {
	TCNT0=0x05; // Reinitialize Timer 0 value

	if (t1kHz == 0) {    // 
		t1kHz = 1;       // (     2)
		count_1_min++;

		if (count_2_sec < 2000) { // 2   
			count_2_sec++;
			PORT_ALARM_1KHZ = 1;
		}
		else {
			count_1_sec++;       // 1   
		}

		if (count_1_sec > 1000) { //  1     
			count_2_sec = 0;
			count_1_sec = 0;
		}
	}
	else {
		t1kHz = 0;
		PORT_ALARM_1KHZ = 0;
	}

	if (count_1_min >= 60000) { // 60000 = 1 
		PORT_ALARM_1KHZ  = 0;
		PORT_ALARM_BIPER = 0;

		TCCR0=0x00; //  1    
		            // .. Timer/Counter 0 Stoped
	}
}//-----------------------------------------------------------------------------------------


// -----         --------------
// 
//          ( 2009   29 , 2010  28 ).
//          ( 2009   25 ,  2010  31 ).
//
void date_LetoZima(void) {
unsigned char y;

	y = bcd2bin(myyear);
		
	//        (ru.wikipedia.org)
	// a = (14 - ) / 12
	// y =  - a
	// m =  + 12 * a - 2
	//  = (7000 + ( + y + y / 4 - y / 100 + y / 400 + (31 * m) / 12))  7

	// __ = 31  __31_
	last_sunday_March   = bin2bcd(31 - ((33 + y + (y>>2)) % 7));

	// __ = 31  __31_
	last_sunday_October = bin2bcd(31 - ((51 + y + (y>>2)) % 7));
} // ---------------------------------------------------------------------------------------


//-----------------    (    ) --------------
unsigned char Zima_or_Leto(void) {

	//    c 3:00  23:00,
	//  c 1:00  23:00      ,
	//  c 1:00  23:00     ,
	//   1:00  23:00      
	if (((mymonth == 0x03) && (mydate == last_sunday_March) && (myhour >= 0x03) && (myhour < 0x23)) 
	   || ((mymonth == 0x03) && (mydate > last_sunday_March) && (myhour > 0x00) && (myhour < 0x23)) 
	      || ((mymonth > 0x03) && (mymonth < 0x10) && (myhour > 0x00) && (myhour < 0x23))
	         || ((mymonth == 0x10) && (mydate < last_sunday_October) && (myhour > 0x00) && (myhour < 0x23))) {
		return LETO;
	}

	//    c 4:00  23:59,
	//  c 2:00  23:59      ,
	//  c 2:00  23:59     ,
	//  c 2:00  23:59     ,
	//   2:00  23:59      
	if (((mymonth == 0x10) && (mydate == last_sunday_October) && (myhour >= 0x04) && (myhour != 0x00)) 
	   || ((mymonth == 0x10) && (mydate > last_sunday_October) && (myhour != 0x00) && (myhour >= 0x02)) 
	      || ((mymonth > 0x10) && (myhour != 0x00) && (myhour >= 0x02))
	         || ((mymonth < 0x03) && (myhour != 0x00) && (myhour >= 0x02))
	            || ((mymonth == 0x03) && (mydate < last_sunday_March) && (myhour != 0x00) && (myhour >= 0x02))) {
		return ZIMA;
	}
    
	return NE_LETO_NE_ZIMA; //          	
}//-----------------------------------------------------------------------------------------


// --------  ()       23:00:00 ------------
void calculate_corect(void) {
	if (Corect >= 60) {hour_k=0x23; min_k=0x00; sec_k=bin2bcd(Corect-60);} //  , .  23:00: -> 23:00:00   
	else              {hour_k=0x22; min_k=0x59; sec_k=bin2bcd(Corect);   } //  , .  22:59: -> 23:00:00
}//-----------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------
void calculate_bl_lcd(void) {
	if ((bl_lcd == BL_TR) || (bl_lcd == BL_OFF) || (bl_lcd == BL_ON)) bl_lcd_on = -1;
	if (bl_lcd == BL_10) bl_lcd_on = 1;
	if (bl_lcd == BL_20) bl_lcd_on = 2;
	if (bl_lcd == BL_30) bl_lcd_on = 3;
	if (bl_lcd == BL_1M) bl_lcd_on = 6;
}//-----------------------------------------------------------------------------------------


// -----------------  ()    DS1307 -------------------
void read_SetUp(void) {
	unsigned char i, j;

	Corect       = rtc_read(MEM_CORECT); //     DS1307
	bl_lcd       = rtc_read(MEM_BL_LCD);
	menu         = rtc_read(MEM_MENU);
	num_term     = rtc_read(MEM_NUM_TERM);
	max_alarm    = rtc_read(MEM_MAX_ALARM);
	flag_al      = rtc_read(MEM_FLAG_AL);
	month_corect = rtc_read(MEM_MONTH_CORECT);


	//   
	if (Corect >= 122)    Corect    = 60;
	if (bl_lcd > 6)       bl_lcd    = BL_30;
	if (menu >= MAX_MENU) menu      = 0;
	if (num_term > 1)     num_term  = 0;
	if (max_alarm > 8)    max_alarm = 0;
	if (month_corect > 1) month_corect = ZIMA;

	calculate_bl_lcd();
	calculate_corect();

	j=MEM_ALARM;
	for (i=0; i<8; i++,j++) {hour_al[i] = rtc_read(j);} // 16 17 18 19 20 21 22 23
	for (i=0; i<8; i++,j++) {min_al[i]  = rtc_read(j);} // 24 25 26 27 28 29 30 31
	for (i=0; i<8; i++,j++) {day_al[i]  = rtc_read(j);} // 32 33 34 35 36 37 38 39

	for (i=0; i<8; i++) {
		if (bcd2bin(hour_al[i]) > 23) {hour_al[i]=0; min_al[i]=0; day_al[i]=0b01111111; flag_al=0;}
	}
}//-----------------------------------------------------------------------------------------


// -------------------  ()    DS1307 ----------------------
void save_SetUp(void) {
	unsigned char i, j;

	rtc_write(MEM_CORECT, Corect);
	rtc_write(MEM_BL_LCD, bl_lcd);
	rtc_write(MEM_MENU, menu);
	rtc_write(MEM_NUM_TERM, num_term);
	rtc_write(MEM_MAX_ALARM, max_alarm);
	rtc_write(MEM_FLAG_AL, flag_al);
	rtc_write(MEM_MONTH_CORECT, month_corect);
	j=MEM_ALARM;
	for (i=0; i<8; i++,j++) {rtc_write(j,hour_al[i]);} // 16 17 18 19 20 21 22 23
	for (i=0; i<8; i++,j++) {rtc_write(j,min_al[i]); } // 24 25 26 27 28 29 30 31
	for (i=0; i<8; i++,j++) {rtc_write(j,day_al[i]); } // 32 33 34 35 36 37 38 39
}//-----------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------
void alarm_ON(void) {
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 500,000 kHz
	TCCR0=0x02; TCNT0=0x05;
	count_1_min = 0;
	count_2_sec = 0;
	count_1_sec = 0;

	PORT_ALARM_BIPER = 1;
}//-----------------------------------------------------------------------------------------


// --------------------      --------------------------------------
void date2day(void) {
unsigned int y, m;

	y = bcd2bin(myyear);
	m = bcd2bin(mymonth);

	if (m > 2) { m -= 2;       }
	else       { m += 10; y--; }

	myday = (unsigned char)((bcd2bin(mydate) + y + (y>>2) + ((31 * m) / 12)) % 7);
	
	if (myday == 0) myday = 7;
}//-----------------------------------------------------------------------------------------


// ----------------   DS1307    DS1307 ----------------
void DS1307_init (void) {
        unsigned char tmpsec1, tmpsec2;

        //rtc_init(0, 0, 0); //    DS1307    
                             // (   - : rs=0, sqwe=0, out=0)
     
        rtc_write(0x07,0x00); //    M41T56 STMicroelectronics 
                              //  ,   "rtc_init(0, 0, 0)"

        //   (   )
        //myhour=0; mymin=0; mysec=0;       // 24-00-00
        //mydate=29; mymonth=06; myyear=10; // 29.06.2010

        tmpsec1 = rtc_read(0x00); //     
        delay_ms(1500);
        tmpsec2 = rtc_read(0x00); //  1,5 .    

        if (tmpsec1 == tmpsec2) { //   1,5 .        
                rtc_set_time(0, 0, 0);  //  7 (CH)   DS1307    
                                        // (0 - , 1 - )
                rtc_set_date(0x29, 0x06, 0x10);
        }
}//-----------------------------------------------------------------------------------------


// -----------------------------    DS1307 ----------------------------------
void read_time(void) {
        rtc_get_time(&myhour, &mymin, &mysec);
        rtc_get_date(&mydate, &mymonth, &myyear);
        date2day(); //     

        if (myhour == 0x22) K=1; //     22 

        //  Corect = 60  = 0 
        if ((K == 1) && (Corect != 60) && (myhour == hour_k) && (mymin == min_k) && (mysec == sec_k)) { 
                myhour=0x23; mymin=0x00; mysec=0x00;
                rtc_set_time(myhour, mymin, mysec);
                K=0;             //        
        }
        
        //     22:59: ( - )      23:00:00
        //   ,   23:00:     23:00:00 
        //          22 
} //----------------------------------------------------------------------------------------


// -----------------------------------------------------------------------------------------
void view_term(int term) {
	unsigned char tmp, celie, drob;
	unsigned int temp, celie_tmp, drob_tmp;

	temp = (unsigned int) term;

	if (term < 0) {
		temp = ( ~temp ) + 0x0001; //       . 
		lcd_putchar('-');          //    
	}
	else lcd_putchar('+');

	celie_tmp = temp >> 4;     //   
	drob_tmp  = temp & 0x000F; //      1/16 

	celie = (unsigned char) celie_tmp;  //     
	drob  = (unsigned char) ((drob_tmp * 10) / 16); //      "1/16 "
	                                                //      0,1  

	if (celie >= 100) { tmp=bin2bcd(celie-100); lcd_putchar('1');} else tmp=bin2bcd(celie); //   

	if (celie_tmp < 100) {   //     100 .  0,1 . 
		if ((tmp >> 4)!= 0)
		lcd_putchar('0' + (tmp >> 4));   // 

		lcd_putchar('0' + (tmp & 0x0F)); // 
		lcd_putchar(',');
		lcd_putchar('0' + drob);         //   
	}
	else {                   //  100 .  1,0 .
		lcd_putchar('0' + (tmp >> 4));   // 
		lcd_putchar('0' + (tmp & 0x0F)); // 
	}
}// ----------------------------------------------------------------------------------------


// -----------------------------       ? -------------------------------
void view_time(void) {
        unsigned char tmp;

        if (menu > MAX_MENU) menu = 0; //      ()    
              
        // ======================================    ===========================================
        if (indicate == IND_NORM) {

                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf(str_1[menu]); //   
                lcd_gotoxy(0,1); lcd_putsf(str_2[menu]); //   

                if (menu_hour[menu] == 1) { //        
                        lcd_gotoxy(menu_hour_x[menu], menu_hour_y[menu]);
                        lcd_putchar('0' + (myhour >> 4));
                        lcd_putchar('0' + (myhour & 0x0F));
                }

                if (menu_min[menu] == 1) { //        
                        lcd_gotoxy(menu_min_x[menu], menu_min_y[menu]);
                        lcd_putchar('0' + (mymin >> 4));
                        lcd_putchar('0' + (mymin & 0x0F));
                }

                if (menu_sec[menu] == 1) { //        
                        lcd_gotoxy(menu_sec_x[menu], menu_sec_y[menu]);
                        lcd_putchar('0' + (mysec >> 4));
                        lcd_putchar('0' + (mysec & 0x0F));
                }

                if (menu_day[menu] == 1) { //         
                        lcd_gotoxy(menu_day_x[menu], menu_day_y[menu]);
                        lcd_putsf(str_day[myday]);
                }
                if (menu_day[menu] == 2) {
                        lcd_gotoxy(menu_day_x[menu], menu_day_y[menu]);
                        lcd_putsf(str_day2[myday]);
                }

                if (menu_date[menu] == 1) { //        
                        lcd_gotoxy(menu_date_x[menu], menu_date_y[menu]);
                        lcd_putchar('0' + (mydate >> 4));
                        lcd_putchar('0' + (mydate & 0x0F));
                }
                
                if (menu_month[menu] == 1) { //        
                        lcd_gotoxy(menu_month_x[menu], menu_month_y[menu]);
                        lcd_putchar('0' + (mymonth >> 4));
                        lcd_putchar('0' + (mymonth & 0x0F));
                }

                if (menu_year[menu] == 1) { //        
                        lcd_gotoxy(menu_year_x[menu], menu_year_y[menu]);
                        lcd_putchar('0' + (myyear >> 4));
                        lcd_putchar('0' + (myyear & 0x0F));
                }

                if ((menu_t1[menu] == 1) && (temperature_1!=-9999)) {  //     .    
                        lcd_gotoxy(menu_t1_x[menu], menu_t1_y[menu]);
                        view_term(temperature_1);
                }

                if ((menu_t2[menu] == 1) && (temperature_2!=-9999)) { //     .    
                        lcd_gotoxy(menu_t2_x[menu], menu_t2_y[menu]);
                        view_term(temperature_2);
                }
        }


        // ==================================    =======================================
        if (indicate == SET_BL) {
                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf(" .  ");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(2,1); lcd_putsf(str_bl[bl_lcd]);
        }


                
        // =======================================   ===========================================
        if (indicate == SET_TIME) {

                lcd_gotoxy(0,0); lcd_putsf("  .   ");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(3,1);
                lcd_putchar('0' + (myhour >> 4));
                lcd_putchar('0' + (myhour & 0x0F));
                lcd_putchar(':');
                lcd_putchar('0' + (mymin >> 4));
                lcd_putchar('0' + (mymin & 0x0F));
                lcd_putchar(':');
                lcd_putchar('0' + (mysec >> 4));
                lcd_putchar('0' + (mysec & 0x0F));

                lcd_gotoxy(cursor,1); lcd_cursor_on();
        }

        // =======================================   ==============================================
        if (indicate == SET_DATE) {

                lcd_gotoxy(0,0); lcd_putsf("    .    ");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(3,1);
                lcd_putchar('0' + (mydate >> 4));
                lcd_putchar('0' + (mydate & 0x0F));
                lcd_putchar('.');
                lcd_putchar('0' + (mymonth >> 4));
                lcd_putchar('0' + (mymonth & 0x0F));
                lcd_putchar('.');
                lcd_putchar('0' + (myyear >> 4));
                lcd_putchar('0' + (myyear & 0x0F)); 
                lcd_putchar(',');
                lcd_putsf(str_day[myday]);

                lcd_gotoxy(cursor,1); lcd_cursor_on();
        }


        // ==================================== -  =============================================
        if (indicate == SET_MAX_AL) {
                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf("- .(0-8)");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(6,1);
                lcd_putchar('0' + max_alarm);
        }


        // ====================================   ==========================================
        if (indicate == SET_ALARM) {
                lcd_gotoxy(0,0); lcd_putsf(".            ");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(4,0); lcd_putchar('0' + (num_alarm+1));
                lcd_putchar(' ');
                lcd_putchar('0' + (hour_al[num_alarm] >> 4));
                lcd_putchar('0' + (hour_al[num_alarm] & 0x0F));
                lcd_putchar(':');
                lcd_putchar('0' + (min_al[num_alarm] >> 4));
                lcd_putchar('0' + (min_al[num_alarm] & 0x0F));
                lcd_putchar(' ');

                if ((day_al[num_alarm] & 0b10000000) == 0) {
                                                             lcd_putsf("");
                } else {
                        if ((flag_al & (1<<num_alarm)) == 0) lcd_putsf("");
                        else                                 lcd_putsf("1");
                     }

                lcd_gotoxy(1,1);
                if ((day_al[num_alarm] & 0b00000001) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b00000010) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b00000100) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b00001000) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b00010000) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b00100000) != 0) lcd_putsf(""); else lcd_putsf("__");
                if ((day_al[num_alarm] & 0b01000000) != 0) lcd_putsf(""); else lcd_putsf("__");

                lcd_gotoxy(cursor,cursor_y); lcd_cursor_on();
        }


        // ==================================     ==================================
        if (indicate == SET_K) {
                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf(" .  ");
                lcd_gotoxy(0,1); lcd_putsf("     ./ ");

                lcd_gotoxy(1,1);

                if (Corect > 60)  { lcd_putchar('-'); tmp = bin2bcd(Corect-60); }
                if (Corect == 60) { lcd_putchar(' '); tmp = 0; }
                if (Corect < 60)  { lcd_putchar('+'); tmp = bin2bcd(60-Corect); }

                lcd_putchar('0' + (tmp >> 4));
                lcd_putchar('0' + (tmp & 0x0F));
        }


        // ================================     ===================================
        if (indicate == SET_TERM) {
                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf("   .  ");
                lcd_gotoxy(0,1); 
                if (num_term == 0) lcd_putsf("  ->   ");
                else               lcd_putsf("  ->   ");
        }


        // ====================================  ( ) =============================================
        if (indicate == SET_VIEW) {
                lcd_cursor_off();

                lcd_gotoxy(0,0); lcd_putsf("           ");
                lcd_gotoxy(0,1); lcd_putsf("                ");

                lcd_gotoxy(6,1);
                lcd_putchar('0' + ((menu+1) >> 4));
                lcd_putchar('0' + ((menu+1) & 0x0F));
        }



} //---------------------------------------------------------------------------------------------------------------------------


// --------------------------------  ADC ---------------------------------------------
// Read the 8 most significant bits of the AD conversion result
unsigned char read_adc(unsigned char adc_input) {
	ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
	// Delay needed for the stabilization of the ADC input voltage
	delay_us(10);
	// Start the AD conversion
	ADCSRA|=0x40;
	// Wait for the AD conversion to complete
	while ((ADCSRA & 0x10)==0);
	ADCSRA|=0x10;
	return ADCH;
}  //---------------------------------------------------------------------------------------


// ----------------------------    ------------------------------------
unsigned char read_key(void) {
        unsigned char i, k1, key_adc;

        k1=0;

        key_adc=read_adc(0);


        for (i=0;i<5;i++){      //   
                delay_ms(5);
                if (~PINB.0) k1++;      // 0 -   , 1 -  
        }

        if                      (key_adc <=   3)  return KEY_L; 
        if ((key_adc >= 124) && (key_adc <= 130)) return KEY_R;
        if ((key_adc >= 167) && (key_adc <= 173)) return KEY_U;
        if ((key_adc >= 188) && (key_adc <= 194)) return KEY_D;
        if ((key_adc >= 201) && (key_adc <= 207)) return KEY_MENU;
        if ((key_adc >= 210) && (key_adc <= 215)) return KEY_ESC;

        if (k1 > 3) return KEY_BL;

        return 0;
} // ---------------------------------------------------------------------------------------


// ---------------------------     -----------------------------------
void set_ind(void) {
    unsigned char i, key, tmp;
    key=read_key();

    if (key != 0) { //    

        count_last_key = 0; //  

        //    ""  
        if ((key == KEY_MENU) && (bl_lcd != BL_OFF)) PORT_BL = 1;

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_TIME) {
                if (key == KEY_R) {
                        cursor += 3;  if (cursor > 10) cursor = 4;
                        return;
                }

                if (key == KEY_L) {
                        cursor -= 3;  if (cursor < 4) cursor = 10;
                        return;
                }

                if (key == KEY_U) {
                        if (cursor == 4) { //     
                                tmp=bcd2bin(myhour);
                                tmp++; if (tmp > 23) tmp=0;          //   +1
                                myhour=bin2bcd(tmp);
                        }

                        if (cursor == 7) { //   
                                tmp=bcd2bin(mymin);
                                tmp++; if (tmp > 59) tmp=0;          //  +1
                                mymin=bin2bcd(tmp);
                        }

                        if (cursor == 10) { //   
                                if (bcd2bin(mysec) > 30) {  //   30 
                                        tmp=bcd2bin(mymin);  //  +1
                                        tmp++;
                                        if (tmp > 59) {  //   59 
                                                mymin=0;
                                                tmp=bcd2bin(myhour);
                                                tmp++;
                                                if (tmp > 23) myhour=0; else myhour=bin2bcd(tmp); //   23  
                                        } else mymin=bin2bcd(tmp);
                                }
                                mysec = 0;                           //  00
                        }

                        rtc_set_time(myhour, mymin, mysec);
                        return;
                }


                if (key == KEY_D) {
                        if (cursor == 4) { //    ( -1) 
                                tmp=bcd2bin(myhour);
                                if (tmp == 0) tmp=23;
                                else          tmp--;
                                myhour=bin2bcd(tmp);
                        }

                        if (cursor == 7) { //    ( -1)
                                tmp=bcd2bin(mymin);
                                if (tmp == 0) tmp=59;
                                else          tmp--;
                                mymin=bin2bcd(tmp);
                        }

                        if (cursor == 10) { //    ( 00)
                                mysec = 0;
                        }

                        rtc_set_time(myhour, mymin, mysec);
                        return;
                }
        }

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_DATE) {
                if (key == KEY_R) {
                        cursor += 3;  if (cursor > 10) cursor = 4;
                        return;
                }

                if (key == KEY_L) {
                        cursor -= 3;  if (cursor < 4) cursor = 10;
                        return;
                }

                if (key == KEY_U) {
                        if (cursor == 4) {                 //    ( +1)
                                tmp=bcd2bin(mydate);
                                tmp++; if (tmp > 31) tmp=1;
                                mydate=bin2bcd(tmp);
                        }

                        if (cursor == 7) {                  //    ( +1)
                                tmp=bcd2bin(mymonth);
                                tmp++; if (tmp > 12) tmp=1;
                                mymonth=bin2bcd(tmp);
                        }

                        if (cursor == 10) {                 //    ( +1)
                                tmp=bcd2bin(myyear);
                                tmp++; if (tmp > 99) tmp=0;
                                myyear=bin2bcd(tmp);
                        }

                        rtc_set_date(mydate, mymonth, myyear);
                        date_LetoZima();
                        date2day();
                        month_corect = Zima_or_Leto();
                        return;
                }


                if (key == KEY_D) {
                        if (cursor == 4) {              //    ( -1)
                                tmp=bcd2bin(mydate);
                                if (tmp == 1) tmp=31;
                                else          tmp--;
                                mydate=bin2bcd(tmp);
                        }

                        if (cursor == 7) {              //    ( -1)
                                tmp=bcd2bin(mymonth);
                                if (tmp == 1) tmp=12;
                                else          tmp--;
                                mymonth=bin2bcd(tmp);
                        }

                        if (cursor == 10) {             //    ( -1)
                                tmp=bcd2bin(myyear);
                                if (tmp == 0) tmp=99;
                                else          tmp--;
                                myyear=bin2bcd(tmp);
                        }

                        rtc_set_date(mydate, mymonth, myyear);
                        date_LetoZima();
                        date2day();
                        month_corect = Zima_or_Leto();
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_MAX_AL) {
                if ((key == KEY_U) || (key == KEY_R)){
                        max_alarm++; if (max_alarm > 8) max_alarm = 0; // -  +1
                        return;
                }

                if ((key == KEY_D) || (key == KEY_L)){
                        if (max_alarm == 0) {max_alarm = 8;}           // -  -1
                        else                {max_alarm--;  }
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_ALARM) {
                if (key == KEY_R) {
                        if (cursor_y == 0) {
                                if (cursor == 12) {cursor_y = 1; cursor = 1;}
                                if (cursor == 10) cursor += 2;
                                if (cursor == 7)  cursor += 3;
                        } else {
                                cursor += 2;
                                if (cursor > 13) {cursor_y = 0; cursor = 7;}
                        }
                        return;
                }


                if (key == KEY_L) {
                        if (cursor_y == 0) {
                                if (cursor == 7) {cursor_y = 1; cursor = 13;}
                                if (cursor == 10) cursor -= 3;
                                if (cursor == 12) cursor -= 2;
                        } else {
                                if (cursor == 1) {cursor_y = 0; cursor = 12;}
                                else             {cursor -= 2;              } 
                        }
                        return;
                }


                if ((key == KEY_U) && (cursor_y == 0)) {
                         if (cursor == 7) { //   
                                tmp=bcd2bin(hour_al[num_alarm]);
                                tmp++; if (tmp > 23) tmp=0;            //   +1
                                hour_al[num_alarm]=bin2bcd(tmp);
                        }

                        if (cursor == 10) { //   
                                tmp=bcd2bin(min_al[num_alarm]);
                                tmp++; if (tmp > 59) tmp=0;            //  +1
                                min_al[num_alarm]=bin2bcd(tmp);
                        }

                        if (cursor == 12) { //   
                                if ((day_al[num_alarm] & 0b10000000) == 0) { //  ""
                                        day_al[num_alarm] |= 0b10000000;       //  "" 
                                        flag_al &= (~(1<<num_alarm));             // "1" = '0'
                                } else {
                                        if ((flag_al & (1<<num_alarm)) == 0) { //  ""
                                             flag_al |= (1<<num_alarm);          //  "1"
                                        } else {                               // 
                                             flag_al &= (~(1<<num_alarm));       // "1" = '0' 
                                             day_al[num_alarm] &= 0b01111111;      // "" = '0' .. ""
                                        }
                                }
                        }

                        return;
                }

                if ((key == KEY_D) && (cursor_y == 0)) {
                        if (cursor == 7) { //   
                                tmp=bcd2bin(hour_al[num_alarm]);
                                if (tmp == 0) tmp=23;            //   -1
                                else          tmp--;
                                hour_al[num_alarm]=bin2bcd(tmp);
                        }

                        if (cursor == 10) { //   
                                tmp=bcd2bin(min_al[num_alarm]);
                                if (tmp == 0) tmp=59;            //  -1
                                else          tmp--;
                                min_al[num_alarm]=bin2bcd(tmp);
                        }

                        if (cursor == 12) { //   
                                if ((day_al[num_alarm] & 0b10000000) == 0) { //  ""
                                        day_al[num_alarm] |= 0b10000000;       //  
                                        flag_al |= (1<<num_alarm);               // "1" = '1'
                                } else {
                                        if ((flag_al & (1<<num_alarm)) == 0) { //  ""
                                             day_al[num_alarm] &= 0b01111111;    //  ""
                                             flag_al &= (~(1<<num_alarm));
                                        } else {                               //  "1"
                                             flag_al &= (~(1<<num_alarm));       // "1" = '0' 
                                             day_al[num_alarm] |= 0b10000000;      // "" = '1'
                                        }
                                }
                        }

                        return;
                }

                if (((key == KEY_U) || (key == KEY_D)) && (cursor_y == 1)) {
                        i = (cursor/2) + 1; tmp = 1 << (cursor/2);

                        if ((day_al[num_alarm] & tmp) == 0) {
                               day_al[num_alarm] |= tmp;    lcd_putsf(str_day[i]);
                        } else {
                               day_al[num_alarm] &= (~tmp); lcd_putsf("__");
                        }
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_BL) {
                if ((key == KEY_U) || (key == KEY_R)){
                        bl_lcd++; if (bl_lcd > 6) bl_lcd = 0;
                        calculate_bl_lcd();
                        return;
                }

                if ((key == KEY_D) || (key == KEY_L)){
                        bl_lcd--; if (bl_lcd < 0) bl_lcd = 6;
                        calculate_bl_lcd();
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_K) {
                //  1  59 -   ()
                //  60  119 -   ()

                if (Corect >= 120) Corect=1; //    

                if ((key == KEY_U) || (key == KEY_R)){
                        Corect++;
                        if (Corect >= 120) Corect=1; //      
                        return;
                }

                if ((key == KEY_D) || (key == KEY_L)){
                        Corect--;
                        if (Corect == 0) Corect=119; //      
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_TERM) {
                if ((key == KEY_U) || (key == KEY_R) || (key == KEY_D) || (key == KEY_L)){
                        if (num_term == 0) num_term = 1;
                        else               num_term = 0;
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (indicate == SET_VIEW) {
                if ((key == KEY_U) || (key == KEY_R)){
                        menu++; if (menu >= MAX_MENU) menu = 0;
                        return;
                }
                
                if ((key == KEY_D) || (key == KEY_L)){
                        if (menu == 0) {menu=MAX_MENU-1;}
                        else           {menu--;         }
                        return;
                }
        }


        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (key == KEY_MENU) { //  "" -     

                //    -   0     
                if ((indicate == SET_MAX_AL) && (max_alarm == 0)) indicate ++;

                //         
                if (indicate == SET_ALARM) {
                    num_alarm++;
                    if (num_alarm >= max_alarm) {num_alarm=0; indicate++;}
                } else {
                    indicate++; if (indicate > SET_VIEW) indicate = IND_NORM;
                }

                //  ..  .. ->   4 
                if ((indicate == SET_TIME) || (indicate == SET_DATE)) cursor = 4; 

                //  .. ->   7 
                if (indicate == SET_ALARM) { cursor = 7; cursor_y = 0; } 
        }


        if (key == KEY_ESC) { //  ."Esc" ->   . 
                indicate = IND_NORM;
                //    
                if (bl_lcd == BL_ON)  PORT_BL = 1;
                if (bl_lcd == BL_OFF) PORT_BL = 0;
                calculate_bl_lcd();
        }


        if ((key == KEY_MENU) || (key == KEY_ESC)) {  //  .""  "Esc" ->     DS1307

                //     DS1307
                save_SetUp();

                // ----------    ------------
                //  ()       23:00:00
                calculate_corect();

                return;
        }

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if (key == KEY_BL) { //    ""
                if (bl_lcd == BL_TR) {
                        if (PIN_BL == 0)  PORT_BL = 1; 
                        else              PORT_BL = 0;
                } 

                if (bl_lcd == BL_10) { PORT_BL = 1; bl_lcd_on = 1; }
                if (bl_lcd == BL_20) { PORT_BL = 1; bl_lcd_on = 2; }
                if (bl_lcd == BL_30) { PORT_BL = 1; bl_lcd_on = 3; }
                if (bl_lcd == BL_1M) { PORT_BL = 1; bl_lcd_on = 6; }

                //     (    )
                PORT_ALARM_1KHZ  = 0;
                PORT_ALARM_BIPER = 0;
                TCCR0=0x00; // Timer/Counter 0 Stoped
        }

    }
} // ---------------------------------------------------------------------------------------


//=============================================== main ==============================================
void main(void) {
unsigned char i, devices, t1, t2, tmp;

#include "mega8.init" //  

w1_init(); // 1 Wire Bus initialization

lcd_init(16); // LCD module initialization

DS1307_init(); //  DS1307

alarm_activ = 0x00; //    

read_SetUp();  //     DS1307

//    
if (bl_lcd == BL_ON)  PORT_BL = 1;
if (bl_lcd == BL_OFF) PORT_BL = 0;

lcd_putsf(" ***  ***  \n");
lcd_putsf("       ");
delay_ms(2000);
lcd_clear();
lcd_putsf("  -=- 2010 -=- \n");
lcd_putsf("   radiokot.ru  ");
delay_ms(1000);

//   ( )
devices=w1_search(0xf0,rom_code);

lcd_clear();
//  2     -  
if (devices == 1) {lcd_putsf(" 1 \n  DS18x20");                            }
else              {lcd_putsf(" "); lcd_putchar(devices + 48); lcd_putchar('\n');}

if ((devices == 0) || (devices >= 5)) {lcd_putsf(" DS18x20");}
else if (devices != 1)                {lcd_putsf(" DS18x20"); }

// ..       
//      
if (devices >= 1) {
	for (i=0;i<devices;i++) {
		temperature_1=ds18x20_temperature(&rom_code[i][0]); 
		delay_ms(1000);
	}
} else {
	delay_ms(2000);
}

lcd_clear();

// Global enable interrupts
#asm("sei")

 // --------------------------------   -----------------------------------
 while (1) {
        //      
        if (indicate == IND_NORM) {
            read_time();
            date_LetoZima();

            //     ,    "" 
            if (month_corect == ZIMA) { 
                if (Zima_or_Leto() == LETO) {
                    tmp = bcd2bin(myhour); tmp++; //     
                    myhour = bin2bcd(tmp);
                    rtc_set_time(myhour, mymin, mysec); 

                    month_corect = LETO;                      //  
                    rtc_write(MEM_MONTH_CORECT,month_corect); //     DS1307
                }
            } 
            //     ,    "" 
            else {
                if (Zima_or_Leto() == ZIMA) {
                    tmp = bcd2bin(myhour); tmp--; //     
                    myhour = bin2bcd(tmp);
                    rtc_set_time(myhour, mymin, mysec); 

                    month_corect = ZIMA;                      //  
                    rtc_write(MEM_MONTH_CORECT,month_corect); //     DS1307
                }
            }
        }

        //    DS1307
        read_time();

        //  24:00    
        if ((myhour == 0) && (mymin == 0) && (alarm_activ != 0)) alarm_activ = 0x00;

        if (indicate == IND_NORM) {
            for (i=0; i < max_alarm; i++) {
                //          
                //      
                if (((day_al[i] & 0b10000000) != 0) && ((day_al[i] & (1<<(myday-1))) != 0)) {
                   if ((myhour == hour_al[i]) && (mymin == min_al[i])) {
                      if ((alarm_activ & (1<<i)) == 0) { //   
                         alarm_activ |= (1<<i); //   ..  .

                         //    "1"       
                         if ((flag_al & (1<<i)) != 0) day_al[i] &= (~(1<<(myday-1))); 

                         alarm_ON();
                         if (((i+3)%2) != 0) {PORT_ALARM_OUT = 1;} //   ->  'ON'
                         else                {PORT_ALARM_OUT = 0;} //     ->  'OFF'
                      }
                   }
                }
            }
        }


        if (num_term == 0) { t1=0; t2=1; } else { t1=1; t2=0; } 
 
        if ((devices >= 1) && (indicate == IND_NORM)) {  //  
                temperature_1=ds18x20_temperature(&rom_code[t1][0]); 
        }

        for (i=0;i<25;i++){ // 5 .   (25 * 0.2 .)
                read_time();
                set_ind();
                view_time();
                delay_ms(200);
        }


        if ((devices >= 2) && (indicate == IND_NORM)) {  //  
                temperature_2=ds18x20_temperature(&rom_code[t2][0]); 
        }


        for (i=0;i<25;i++){ // 5 .   (25 * 0.2 .) 
                read_time();
                set_ind();
                view_time();
                delay_ms(200);
        }


        //    6           
        count_last_key++;
        if (count_last_key == 6) {
            save_SetUp();
            indicate = IND_NORM;
            calculate_bl_lcd();
            if (bl_lcd == BL_OFF) PORT_BL = 0;
        }
        if (count_last_key >= 6) {count_last_key = 7;}


        //     
        bl_lcd_on--;
        if (bl_lcd_on == 0) PORT_BL = 0;
        if (bl_lcd_on < 0) bl_lcd_on = -1;

 } // -----------------------------   (end) -----------------------------------

   
 
} // =================================== end main ==================================================





//
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= The End -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//